
#include <cmsis_os.h>                                           // CMSIS RTOS header file
#include <stdio.h>

/*----------------------------------------------------------------------------
 *      Timer: Sample timer functions
 *---------------------------------------------------------------------------*/
 
extern osThreadId tid_Thread;

struct clock {                           
  unsigned char    hour;                                              
  unsigned char     min;                                              
  unsigned char     sec;                                              
  unsigned short   msec;                                       
}; 
static struct clock time;

/*----- Periodic Timer Example -----*/
static void Timer1_Callback  (void const *arg);           // prototype for timer callback function

static osTimerId id1;                                     // timer id
static uint32_t  exec1;                                   // argument for the timer call back function
static osTimerDef (Timer1, Timer1_Callback);              // define timers

// One-Shoot Timer Function
static void Timer1_Callback  (void const *arg)  {
  // add user code here
	osSignalSet(tid_Thread,0x01);
}	
	

/*----- Periodic Timer Example -----*/
static void Timer2_Callback  (void const *arg);           // prototype for timer callback function

static osTimerId id2;                                     // timer id
static uint32_t  exec2;                                   // argument for the timer call back function
static osTimerDef (Timer2, Timer2_Callback);                                      
 
// Periodic Timer Example
static void Timer2_Callback  (void const *arg)  {
	if (++time.msec == 100) {          
    time.msec = 0;
    if (++time.sec == 60) {                 
      time.sec = 0;
      if (++time.min == 60) {                
        time.min = 0;
        if (++time.hour == 24) {              
          time.hour = 0;
        }
      }
    }
  } 
}	


	
// Example: Create and Start timers
void Init_Timers (void) {
  osStatus  status;                                        // function return status
 
  // Create periodic timer
  exec1 = 1;
  id1 = osTimerCreate (osTimer(Timer1), osTimerPeriodic, &exec1);
  if (id1 != NULL)  {     // Periodic timer created
    // start timer with delay 100ms
		status = osTimerStart (id1, 100);            
    if (status != osOK)  {
      // Timer could not be started
    }
	}
 
  // Create periodic timer
  exec2 = 2;
  id2 = osTimerCreate (osTimer(Timer2), osTimerPeriodic, &exec2);
  if (id2 != NULL)  {     // Periodic timer created
    // start timer with periodic 10ms interval
		status = osTimerStart (id2, 10);            
    if (status != osOK)  {
      // Timer could not be started
    }
  }
}

// Print current time			     
void PrintTimeStamp(FILE *f) {
		 fprintf (f,"%02d:%02d:%02d.%02d",
           time.hour,           
           time.min,            
           time.sec,             
           time.msec);          
}
